@@ -1,7 +1,7 @@
 <?php
 /* vim: set expandtab sw=4 ts=4 sts=4: */
 /**
- ** Test for PMA_Util::formatSql from Util.class.php
+ ** Test for PMA\libraries\Util::formatSql from Util.php
  *
  * @package PhpMyAdmin-test
  * @group common.lib-tests
@@ -10,11 +10,10 @@
 /*
  * Include to test.
  */
-require_once 'libraries/Util.class.php';
-require_once 'libraries/sqlparser.lib.php';
+
 
 /**
- ** Test for PMA_Util::formatSql from Util.class.php
+ ** Test for PMA\libraries\Util::formatSql from Util.php
  *
  * @package PhpMyAdmin-test
  * @group common.lib-tests
@@ -33,7 +32,7 @@ class PMA_FormatSql_Test extends PHPUnit_Framework_TestCase
             '<code class="sql"><pre>' . "\n"
             . 'SELECT 1 &lt; 2' . "\n"
             . '</pre></code>',
-            PMA_Util::formatSql('SELECT 1 < 2')
+            PMA\libraries\Util::formatSql('SELECT 1 < 2')
         );
     }
 
@@ -50,7 +49,7 @@ class PMA_FormatSql_Test extends PHPUnit_Framework_TestCase
             '<code class="sql"><pre>' . "\n"
             . 'SELECT[...]' . "\n"
             . '</pre></code>',
-            PMA_Util::formatSql('SELECT 1 < 2', true)
+            PMA\libraries\Util::formatSql('SELECT 1 < 2', true)
         );
     }
 }
